/* Select Network dialogs */
.nm-dialog {
  max-height: 34em;
  min-height: 31em;
  min-width: 32em;
}

.nm-dialog-content {
  spacing: 20px;
  padding: 24px;
}

.nm-dialog-header { @extend %heading; }
.nm-dialog-subheader { color: $insensitive_fg_color;}
.nm-dialog-header-icon { icon-size: $large_icon_size;}
.nm-dialog-header-hbox { spacing: 10px; }

.nm-dialog-network-selected {
  margin: 0 $base_margin*2;
}

.nm-dialog-scroll-view {
  padding:$base_padding;
  border-radius: $base_border_radius;
  background-color: $base_color;
}

.nm-dialog-item {
  padding: $base_padding * 2;

  &:selected {
    background-color: $selected_bg_color;
    border-radius: $base_border_radius - 3px;
    color: $selected_fg_color;
  }
  &:hover { background-color:$hover_bg_color;}
}

.nm-dialog-icon { icon-size: $base_icon_size; }
.nm-dialog-icons { spacing: $base_padding * 2; }

// no networks
.no-networks-box { spacing: $base_padding; }
.no-networks-label { color: $insensitive_fg_color; }

// airplane mode
.nm-dialog-airplane-box {
  text-align: center;
  spacing: 12px;
}
.nm-dialog-airplane-headline { @extend %title_3;}
.nm-dialog-airplane-text { color: $insensitive_fg_color;}
